;;; dino-cc-mode-init.el --- Configuration of the dino c, c++ environment 
;;	$Id: dino-cc-mode-init.el,v 1.29 2005/02/07 17:10:55 sthalau Exp $	

;; Copyright (C) 2003  IICM, Graz University of Technology
;; Inffeldgasse 16c, A-8010 Graz, Austria.

;; Author: stefan thalauer  <sthalau@iicm.edu>
;; Keywords: c, languages, oop, internal


;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Usage:

;; Copy this file in your ~/lib/lisp directory

;; Copy the file `dino-skeletons.el' and `dino-cc-mode-skeletons.el' in your ~/lib/lisp directory

;; Put this in your ~/.emacs file:

;; ;; that's the place where private lisp stuff resides...
;; (add-to-list 'load-path "~/lib/lisp")
;;
;; (if (not (load "dino-cc-mode-init" t nil nil t)) ; load the c, c++ mode configuration file
;; 		(message "File dino-cc-mode-init.el[c] not exist ")
;; 	)

;;; Code:

;;--------------------------------------- all necessary requires
(require 'font-lock)
(require 'autoinsert)
(require 'dino-cc-mode-skeletons)


;;----------------------------------------------------------------------
;; Auto-Mode Customization
;;----------------------------------------------------------------------
(mapcar
 '(lambda (auto)
		(setq auto-mode-alist
          (cons auto
                (delq (assoc (car auto) auto-mode-alist)
                      auto-mode-alist))))
 '(("\\.c\\'" 	 .  c-mode   )
	 ("\\.h\\'" 	 .  c++-mode )
	 ("\\.cpp\\'"  .  c++-mode )))


;;----------------------------------------------------------------------
;; Common Settings for C and C++ Mode
;;----------------------------------------------------------------------
(add-hook 'c-mode-common-hook 'turn-on-auto-fill) ; enable line wrap
(add-hook 'c-mode-common-hook 'turn-on-font-lock)	; enable syntax highlighting
(add-hook 'c-mode-common-hook
          (function
           (lambda ()
             (setq default-tab-width 2
									 indent-tabs-mode nil 
									 c-tab-always-indent t
									 transient-mark-mode t
									 comment-column 40 
                   fill-column 80
                   )
             )))

;;----------------------------------------------------------------------
;; Skeleton Stuff
;;----------------------------------------------------------------------
(setq dino-skeletons-makefile-default-compiler "g++"
			dino-skeletons-use-copyright t
			dino-skeletons-use-cvs-header t
			)

; FIXXME (Stefan Thalauer, 2004-12-13 14:46:26) -> Removed expand 

;; (defconst dino-c++-expand-list
;;   '(
;;     ("dif" dino-if-skeleton)
;;     ("dife"dino-if-else-skeleton)
;;     ("dforc" dino-for-count-skeleton)
;;     ("dfix" dino-fixxme-skeleton)
;;     ("dclass" dino-c++-header-class-skeleton)
;;     ("dnas" dino-namespace-skeleton)
;;     )
;;   "Dino expansions for CC mode")

;; (add-hook 'c-mode-common-hook (function (lambda ()
;; 				   (expand-add-abbrevs c++-mode-abbrev-table dino-c++-expand-list))))

; END FIXXME (Stefan Thalauer, 2004-12-13 14:46:26)

(define-abbrev-table 'c-mode-abbrev-table
  '(("else"    "else"       c-electric-continued-statement 0)
    ("fixx"    ""           dino-fixxme-skeleton 0)
    ("forb"    ""           dino-for-skeleton 0)
    ("ifb"     ""           dino-if-skeleton 0)
    ("ifdef"   ""           dino-ifdef-skeleton 0)
    ("ife"     ""           dino-if-else-skeleton 0)
    ("inc"     ""           dino-include-file--skeleton 0)
    ("incc"    ""           dino-include-c-skeleton 0)
    ("mainf"   ""           dino-main-function-skeleton 0)
    ("switchb" ""           dino-switch-skeleton 0)
    ("typd"    ""           dino-typedef-skeleton 0)
    ("while"   "while"      c-electric-continued-statement 0)
    ("whileb"  ""           dino-while-skeleton 0)))

(define-abbrev-table 'c++-mode-abbrev-table
  '(("autoptr" "auto_ptr"   nil 0)
    ("catch"   "catch"      c-electric-continued-statement 0)
    ("classd"  ""           dino-class-skeleton 0)
    ("del"     "delete"     nil 0)
    ("doutd"   ""           dino-out-debug-skeleton 0)
    ("doute"   ""           dino-out-error-skeleton 0)
    ("douti"   ""           dino-out-info-skeleton 0)
    ("doutw"   ""           dino-out-warning-skeleton 0)
    ("else"    "else"       c-electric-continued-statement 0)
    ("fixx"    ""           dino-fixxme-skeleton 0)
    ("forb"    ""           dino-for-skeleton 0)
    ("ifb"     ""           dino-if-skeleton 0)
    ("ifdef"   ""           dino-ifdef-skeleton 0)
    ("ife"     ""           dino-if-else-skeleton 0)
    ("inc"     ""           dino-include-file-nondirectory-skeleton 0)
    ("incace"  ""           dino-include-ace-skeleton 0)
    ("incstl"  ""           dino-include-stl-skeleton 0)
    ("incutil" ""           dino-include-util-skeleton 0)
    ("mainf"   ""           dino-main-function-skeleton 0)
    ("nsd"     ""           dino-namespace-skeleton 0)
    ("nsdi"    ""           dino-namespace-dino-skeleton 0)
    ("pr"      "private"    nil 0)
    ("pro"     "protected"  nil 0)
    ("pu"      "public"     nil 0)
    ("switchb" ""           dino-switch-skeleton 0)
    ("typd"    ""           dino-typedef-skeleton 0)
    ("while"   "while"      c-electric-continued-statement 0)
    ("whileb"  ""           dino-while-skeleton 0)))



;;----------------------------------------------------------------------
;; Font Lock Mode (Text Highlighting)
;;----------------------------------------------------------------------

;; font highlighting of user types
(add-to-list 'c++-font-lock-extra-types "int8"   t)
(add-to-list 'c++-font-lock-extra-types "uint8"  t)
(add-to-list 'c++-font-lock-extra-types "int16"  t)
(add-to-list 'c++-font-lock-extra-types "uint16" t)
(add-to-list 'c++-font-lock-extra-types "int32"  t) 
(add-to-list 'c++-font-lock-extra-types "uint32" t) 
(add-to-list 'c++-font-lock-extra-types "int64"  t)
(add-to-list 'c++-font-lock-extra-types "uint64" t)

;; font highlighting of "FIXXME"  
(font-lock-add-keywords
 'c-mode
 '(("\\<\\(FIX+ME\\)\\>" 1 font-lock-warning-face t)))

(font-lock-add-keywords
 'c++-mode
 '(("\\<\\(FIX+ME\\)\\>" 1 font-lock-warning-face t)))

;; Highlight Struct (Classes) : AllWordsCapitalizedWithoutUnderscores
(add-to-list 'c-font-lock-extra-types "\\([A-Z][a-z]+\\)+" t) 
(add-to-list 'c++-font-lock-extra-types "\\([A-Z]+[a-z]+\\)+[A-Z0-9]*" t)
(add-to-list 'c++-font-lock-extra-types "GUH" t)
(add-to-list 'c++-font-lock-extra-types "ACE_[A-Za-z_]*" t)


;; font highlighting for Doxygen comments
(if (not (require 'doxymacs "doxymacs" t)) ;doxymac not available
		(font-lock-add-keywords
		 'c++-mode
		 '(("\\s \\(@\\w+\\)" 1 font-lock-doc-face t))) ;( @<word>)
	;;else
	(font-lock-add-keywords
	 'c-mode doxymacs-doxygen-keywords)

	(font-lock-add-keywords
	 'c++-mode doxymacs-doxygen-keywords)
	)

;; Let c-fill-paragraph recognize doxygen markup as paragraph starters.
(setq	c-append-paragraph-start "\\(@[a-zA-Z]+\\>\\|$\\)")

;; Funktion dino-function which calls first the skeleton:
;; dino-function-declaration 
;; and then
;; dino-doxymacs-C++-function-comment
;; Attentions: this function is only available if doxymacs is installed 

(if (fboundp 'doxymacs-find-next-func)
		(defun dino-function ()
			"Inserts a function with comments"
			(interactive)
			(save-excursion
				(dino-function-declaration-skeleton)
				)
			(dino-doxymacs-C++-function-comment-skeleton "<Description>")
	))


;;----------------------------------------------------------------------
;; C Mode
;;----------------------------------------------------------------------

(add-hook 'c-mode-hook
          (function
           (lambda ()
             (setq comment-start "//"
									 comment-end ""
									 c-comment-start-regexp c-C++-comment-start-regexp
									 ))))


;; compile command for c 
;; usefull for "Einfuehrung in die Strukturierte Programmierung"
(add-hook 'c-mode-hook
          (lambda ()
            (unless (or (file-exists-p "makefile")
                        (file-exists-p "Makefile"))
              (set (make-local-variable 'compile-command)
                   (concat "gcc " buffer-file-name
                           " -o " (file-name-sans-extension buffer-file-name)
                           " -Wall"
                           )))
            ))

;;----------------------------------------------------------------------
;; C++ Mode
;;----------------------------------------------------------------------
;; (setq dino-makefile-default-compiler "g++")	;set default compiler for makefile auto insert
(defvar dino-c++-ace-use nil "If t the string \"-I $ACE_ROOT -L $ACE_ROOT\\ace\\ -l ACE\" is appended to the compile-command")
(defvar dino-confix-use t)
(defvar dino-confix-script "confix" "shell command to invoke confix")
(defvar dino-confix-profile "" "confix default package profile

Possible Package Profiles: util, dino, esrm, ex, amm, nmm, omm, imm, vsm, ka")
(defvar dino-confix-command "all" "confix default command

Possible commands: all, init, clean, delete, delete_install, make, compile, install, uninstall, boo, booc, bootstrap, bootstrap_conf, update")

(defun dino-set-compile-command ()
	"Sets the compile command"
;; 	(if dino-confix-use
	(if (file-exists-p "Makefile.py")
			(set (make-local-variable 'compile-command)
					 (concat dino-confix-script  " " 
									 dino-confix-profile " "
									 dino-confix-command)) 
		(unless (or (file-exists-p "makefile")
								(file-exists-p "Makefile"))
			(set (make-local-variable 'compile-command)
					 (concat "g++ " buffer-file-name
									 " -o " (file-name-sans-extension buffer-file-name)
									 " -Wall"
                   (if dino-c++-ace-use
                       (concat " -I " (file-name-as-directory (getenv "ACE_ROOT"))
                               " -L " (file-name-as-directory (concat (file-name-as-directory 
                                                                       (getenv "ACE_ROOT")) 
                                                                      "ace"))
                               " -l ACE"))
									 )))
		))

(add-hook 'c++-mode-hook 'dino-set-compile-command)


;;----------------------------------------------------------------------
;; Indentation Style
;;----------------------------------------------------------------------
;; electric cc-mode indents
(defconst dino-cc-style
  '((c-basic-offset . 2)
;; FIXXME (Stefan Thalauer, 2003-12-01 13:56:40) -> which indention rule for comment lines
		(c-comment-only-line-offset . 0) 		; put here -1000
;; END FIXXME (Stefan Thalauer, 2003-12-01 13:56:40)
		(c-hanging-braces-alist . (
															 (substatement-open before after)
															 (statement before after)
															 (block-open before after)
															 (brace-list-open before after)
															 (brace-entry-open before after)
															 (block-close . c-snug-do-while)
															 (extern-lang-open before after)
															 (inexpr-class-open before after)
															 (inexpr-class-close before after)
															 )
														)
		(c-offsets-alist .
										 ((string . -1000)
											(c . c-lineup-comment)
											(defun-block-intro . +)
											(class-open . 0)
											(class-close . 0)
											(inline-open . 0)
											(inline-close . 0)
											(knr-argdecl-intro . +)
											(knr-argdecl . 0)
											(topmost-intro . 0)
											(topmost-intro-cont . 0)
											(member-init-intro . +)
											(member-init-cont . 0)
											(inher-intro . +)
											(inher-cont . c-lineup-multi-inher)
											(block-open . -)
											(block-close . 0)
											(brace-list-open . 0)
											(brace-list-close . 0)
											(brace-list-intro . 0)
											(brace-list-entry . 0)
											(statement . c-lineup-runin-statements)
											(statement-cont . c-lineup-math)
											(statement-block-intro . +)
											(statement-case-intro . +)
											(substatement . +)
											(substatement-open . 0)
											(case-label . 0)
											(access-label . -)
											(label . 0)
											(do-while-closure . 0)
											(else-clause . 0)
											(comment-intro . c-lineup-comment)
											(arglist-intro . +)
											(arglist-cont . 0)
											(arglist-cont-nonempty . c-lineup-arglist)
											(arglist-close . +)
											(stream-op . c-lineup-streamop)
											(inclass . +)
											(cpp-macro . -1000)
											(friend . 0)))
;; 		(c-echo-syntactic-information-p . t) ; for debugin only
		) "Dino C/C++ Programming Style")

(add-hook 'c-mode-common-hook
          (function
           (lambda ()
             (c-add-style "dino" dino-cc-style t))))

;;----------------------------------------------------------------------
;; Compile Settings
;;----------------------------------------------------------------------

(setq compilation-window-height 10
      compilation-scroll-output t
      )

;;----------------------------------------------------------------------
;; Makefile Settings
;;----------------------------------------------------------------------
(add-hook 'makefile-mode-hook 'turn-on-font-lock)

;;----------------------------------------------------------------------
;; Autoinserting text in empty files
;;----------------------------------------------------------------------
;; `M-x auto-insert' will put some predefined text at the beginning of the
;; buffer.


(add-hook 'find-file-hooks 'auto-insert) ; insert default contents into new files

;; ;; remove multiple definition for C / C++
;; (setq auto-insert-alist 
;; 			(delq (assoc '("\\.\\([Cc]\\|cc\\|cpp\\)\\'" . "C / C++ program") auto-insert-alist)
;; 						auto-insert-alist))

;; ;; Add Defaults for a C program
;; (add-to-list 'auto-insert-alist '(("\\.c$" . "C program")
;; 																	nil
;; 																	(dino-filename-shortdescription)
;; ))

;; ;; Add Defaults for a C program
;; (add-to-list 'auto-insert-alist '(("\\.cpp$" . "C++ program")
;; 																	nil
;; 																	(dino-filename-shortdescription)
;; ))

(mapcar
 '(lambda (auto)
		(setq auto-insert-alist
          (cons auto
                (delq (assoc (car auto) auto-insert-alist)
                      auto-insert-alist))))
 '(
	(("\\.\\([Hh]\\|hh\\|hpp\\)\\'" . "C / C++ header")
	 nil 
	 (dino-cc-header-skeleton)
	 )
   (("\\.\\([Cc]\\|cc\\|cpp\\)\\'" . "C / C++ program")
    nil       
		(dino-file-shortdescription-skeleton) \n \n
		(if dino-skeletons-use-copyright (dino-copyright-skeleton)) & \n & \n & \n
		)
    (("[Mm]akefile\\'" . "Makefile")
		nil
		(dino-makefile-skeleton)
		)
		))

;;----------------------------------------------------------------------
;; Speedbar
;;----------------------------------------------------------------------
;; (autoload 'speedbar-frame-mode "speedbar" "Popup a speedbar frame" t)
(autoload 'speedbar-get-focus "speedbar" "Jump to speedbar frame" t)

(defvar dino-use-speedbar t)
(if (and (eq window-system 'x) dino-use-speedbar)
		(speedbar 1))

;;----------------------------------------------------------------------
;; Keybindings
;;----------------------------------------------------------------------
(add-hook 'c-mode-common-hook
          (lambda ()
            (local-set-key [f4] 'speedbar-get-focus)
						(local-set-key "\C-cc" 'compile)
            (local-set-key "\C-cC" 'recompile)
						(local-set-key "\M-\"c" 'dino-minus-skeleton)
						(local-set-key "\C-ct" 'dino-timestamp-skeleton)
						(local-set-key "\C-cd" 'dino-date-skeleton)
						(local-set-key "\C-cf" 'dino-fixxme-skeleton)
						(local-set-key "\C-cn" 'dino-namespace-skeleton)
						(define-key c-mode-base-map "\C-m" 'c-context-line-break)
						))


;;----------------------------------------------------------------------
;; Gimicks (not activated yet)
;;----------------------------------------------------------------------

;; (setq-default c-auto-newline t          ; Auto-newline minor mode 
;;               c-hungry-delete-key t     ; Hungry-deletion of Whitespace
;;               )

;;; dino-cc-mode-init.el ends here
