;;; dino-cc-mode-skeletons.el --- Dino Skeletons for the C/C++ Programming Mode

;	$Id: dino-cc-mode-skeletons.el,v 1.21 2005/02/18 14:10:03 sthalau Exp $	

;; Copyright (C) 2005  IICM, Graz University of Technology, Austria

;; Author: stefan thalauer <sthalau@iicm.edu>
;; Keywords: wp ,local, c

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'dino-skeletons)

(defvar dino-skeletons-makefile-default-compiler "gcc")
(defvar dino-skeletons-use-copyright nil)
(defvar dino-skeletons-use-cvs-header nil)
(defvar dino-skeletons-doxygen-group nil)

(defvar dino-include-stl-alist
  '(("algorithm" . nil)
    ("bitset" . nil)
    ("cassert" . nil)
    ("cctype" . nil)
    ("cerrno" . nil)
    ("cfloat" . nil)
    ("ciso646" . nil)
    ("climits" . nil)
    ("clocale" . nil)
    ("cmath" . nil)
    ("complex" . nil)
    ("csetjmp" . nil)
    ("csignal" . nil)
    ("cstdarg" . nil)
    ("cstddef" . nil)
    ("cstdio" . nil)
    ("cstdlib" . nil)
    ("cstring" . nil)
    ("ctime" . nil)
    ("cwchar" . nil)
    ("cwctype" . nil)
    ("deque" . nil)
    ("exception" . nil)
    ("fstream" . nil)
    ("functional" . nil)
    ("hash_map" . nil)
    ("hash_set" . nil)
    ("iomanip" . nil)
    ("ios" . nil)
    ("iosfwd" . nil)
    ("iostream" . nil)
    ("istream" . nil)
    ("iterator" . nil)
    ("limits" . nil)
    ("list" . nil)
    ("locale" . nil)
    ("map" . nil)
    ("memory" . nil)
    ("new" . nil)
    ("numeric" . nil)
    ("ostream" . nil)
    ("queue" . nil)
    ("set" . nil)
    ("slist" . nil)
    ("sstream" . nil)
    ("stack" . nil)
    ("stdexcept" . nil)
    ("streambuf" . nil)
    ("string" . nil)
    ("strstream" . nil)
    ("typeinfo" . nil)
    ("utility" . nil)
    ("valarray" . nil)
    ("vector" . nil)) 
  "Alist for dino-include-stl-skeleton")

(defvar dino-include-util-alist
			'(
        ("ace.h"     . nil)
        ("debug_helper.h"   . nil)
        ("dino_exception.h" . nil)
        ("dino_types.h"   . nil)
        ("generalized_functor.h"     . nil)
        ("ifc_combination.h"     . nil)
        ("invalid_argument_exception.h"     . nil)
        ("log4cpp.h"        . nil)
        ("typelist.h"     . nil)
				) "Alist for dino-include-utils-skeleton")

(defvar dino-podtypes-alist
			'(
				("int8"  . nil)
				("uint8" . nil)
				("int16" . nil)
				("uint16". nil)
				("int32" . nil)
				("uint32". nil)
				("int64" . nil)
				("uint64". nil)
				) "Alist for dino-podtypes-skeleton")

(defvar dino-stl-container-alist
'(
  ("vector"    . nil)
  ("deque"     . nil)
  ("list"      . nil)
  ("set"       . nil)
  ("multiset"  . nil)
  ("map"       . nil)
  ("multimap"  . nil)
  ) "Alist for dino-iterate-stl-container-skeleton")


;;**********************************************************************
;; Autoinsert  
;;**********************************************************************

;;----------------------------------------------------------------------
(define-skeleton dino-cc-header-skeleton
  "Inserts a template for a c++ header"
	nil
	(dino-file-shortdescription-skeleton) \n \n
	(if dino-skeletons-use-cvs-header 
      (concat comment-start " $" "Id" "$") ;; just to cheat cvs 
    ) & \n & \n
 	(if dino-skeletons-use-copyright (dino-copyright-skeleton))
 	'(setq v1 (file-name-nondirectory buffer-file-name))
	'(setq v2 (upcase (concat (substring v1 0 (string-match "\\.[^.]*$" v1))
										"_"
										(substring v1 (1+ (string-match "\\.[^.]*$" v1)))
										"_"
										)))
	"#ifndef " v2  \n
	"#define " v2 \n \n
	_
	\n \n 
	"#endif " comment-start v2
	)

;;----------------------------------------------------------------------
(define-skeleton dino-makefile-skeleton 
	"Inserts a Makefile"
	"Compiler? (gcc or. g++) "
	'(setq input dino-skeletons-makefile-default-compiler) 
	"OBJS = "  \n 
  "CC = " str \n
	"LD = " str \n
	"EXTRA_CCINCLUDES = 
CC_FLAGS = -c $(EXTRA_CCINCLUDES) -Wall
EXTRA_LIBDIRS = 
EXTRA_LIBS = 
EXECUTABLE = 
LD_FLAGS = -o $(EXECUTABLE) $(EXTRA_LIBDIRS) $(EXTRA_LIBS)

$(EXECUTABLE) : $(OBJS)
	$(LD) $(LD_FLAGS) $(OBJS)" \n \n
 _ \n \n
"all : $(EXECUTABLE)

clean : 
	rm $(OBJS) $(EXECUTABLE)")

;######################################################################
;; c/c++ skeletons
;######################################################################

;;**********************************************************************
;; Namespace Skeletons 
;;**********************************************************************

(define-skeleton dino-namespace-skeleton
	"Inserts a namespace declartion"
	nil
	'(setq v1 (upcase-initials (skeleton-read "Namespace: ")))
	'(if (not (string-match "_$" v1)) 
			 (setq v1 (concat v1 "_")))
	> "namespace " v1 \n
  "{" '(c-indent-command) \n
  > _ \n 
  "} " comment-start "end namespace " v1  '(c-indent-command) \n
)

;;---------------------------------------------------------------------
(define-skeleton dino-namespace-dino-skeleton
	"Inserts the namespace dino declaration"
	nil
	'(setq v1 "Dino_")
	> "namespace " v1 \n
  "{" '(c-indent-command) \n
  > _ \n 
  "} " comment-start "end namespace " v1  '(c-indent-command) \n
)

;;---------------------------------------------------------------------
(define-skeleton dino-using-skeleton
"Inserts a using statement"
"Namespace: "
> "using " str ";" \n)

;;---------------------------------------------------------------------
(define-skeleton dino-using-namespace-skeleton
"Inserts a using namespace statement"
"Namespace: "
> "using namespace " str ";" \n)


;;---------------------------------------------------------------------
(define-skeleton dino-using-iostream-skeleton
  "Inserts a using statement, prompts for std::cout, std::cin, std::endl, std::cerr"
  nil
  > "using " 
	(skeleton-read '(completing-read
                   "Using: " 
                   '(
                     ("std::cout" . nil)
                     ("std::cin"  . nil)
                     ("std::cerr" . nil)
                     ("std::endl" . nil)
                     )
                   nil nil "std::"))
  ";" \n)

;;----------------------------------------------------------------------
(define-skeleton dino-podtypes-skeleton
	"Inserts: Dino_::PodTypes_::<pod_type>" 
	nil 
	"Dino_::PodTypes_::"
	(skeleton-read '(completing-read
						    "Type: " dino-podtypes-alist
											 )))


;----------------------------------------------------------------------
(define-skeleton dino-ifdef-skeleton
  "Inserts a c/c++ #ifdef directive"
  nil
  '(setq v1 (upcase (skeleton-read "What defined?: ")))
   "#ifdef " v1 '(c-indent-command) \n
   _ \n 
   "#endif " comment-start v1 '(c-indent-command) \n
   )


(define-skeleton dino-conditional-expression-skeleton
  "Inserts a c/c++ conditional-expression"
  "Expression: "
  str
  " ? " (skeleton-read "True: ")
  " : " (skeleton-read "False: ")
  ";" ?\n
  )


;;----------------------------------------------------------------------
(define-skeleton dino-typedef-skeleton
	"Inserts a new typedef statement" 
	nil 
	"typedef "
	(skeleton-read "Old Type: " ) " "
	(skeleton-read "New Type: " )
	";" ?\n
  )

;;----------------------------------------------------------------------
(define-skeleton dino-singleton-typedef-skeleton
	"Inserts a singleton typedef statement" 
	"Class: "
	"typedef Loki::SingletonHolder<" str "> Singleton;" ?\n
  )

;;**********************************************************************
;; Include directives 
;;**********************************************************************

(define-skeleton dino-include-file-skeleton
	"Inserts a c/c++ include preprocessor directive"
	nil
	"#include \""
	(file-relative-name (read-file-name "Include File"))
	"\"" ?\n
)

;;---------------------------------------------------------------------
(define-skeleton dino-include-file-nondirectory-skeleton
	"Inserts a c/c++ include preprocessor directive without directory"
	nil
	"#include \""
	(file-name-nondirectory (read-file-name "Include File"))
	"\"" ?\n
)

;;---------------------------------------------------------------------
(define-skeleton dino-include-ace-skeleton
	"Inserts a C++ ACE preprocessor directive"
	nil
	"#include \""
	(file-relative-name (read-file-name "Include File: " (file-name-as-directory (concat (file-name-as-directory (getenv "ACE_ROOT")) "ace")))
                      (file-name-as-directory (getenv "ACE_ROOT")))
	"\"" ?\n
  )


;;---------------------------------------------------------------------
(define-skeleton dino-include-stl-skeleton
	"Inserts a C++ STL preprocessor directive"
	nil
	"#include <"
	(skeleton-read '(completing-read
                   "STL Header: " dino-include-stl-alist
											 ))
	">" ?\n
  )

;;---------------------------------------------------------------------
(define-skeleton dino-include-util-skeleton
	"Inserts a C++ Dino Util preprocessor directive"
	nil
	"#include \""
	(skeleton-read '(completing-read
                   "Dino Util Header: " dino-include-util-alist
											 ))
	"\"" ?\n
  )

;;**********************************************************************
;; CC Statements
;;**********************************************************************

;;----------------------------------------------------------------------
(define-skeleton dino-for-skeleton
  "Inserts a for statement"
  nil
  > "for(" _ ";;)" \n
  "{" '(c-indent-command) \n
  \n 
  "}" '(c-indent-command) \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-for-count-skeleton
  "Inserts a for statement"
  "Count lesser then: "
  > "for(int count=0; count < " str "; count++)" \n
  "{" '(c-indent-command) \n
  > _ \n 
  "}" '(c-indent-command) \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-while-skeleton
  "Inserts a while statement"
  nil
  > "while(" _ ")" \n
  "{" '(c-indent-command) \n
  >  \n 
  "}" '(c-indent-command) \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-switch-skeleton
  "Inserts a switch statement"
  nil
  > "switch(" _ ")" \n
  "{" '(c-indent-command) \n
  > @ \n
  >  "default: " ?\n
  > \n
 "}" '(c-indent-command) \n
)


;----------------------------------------------------------------------
(define-skeleton dino-if-skeleton
  "If statement skeleton"
  nil
  > "if("  _ @ ")" \n
  "{" '(c-indent-command) \n
  \n 
  "}" '(c-indent-command) \n
)

;----------------------------------------------------------------------
(define-skeleton dino-if-else-skeleton
  "If else statement skeleton"
  nil
  > "if("  _ @ ")" \n
  "{" '(c-indent-command) \n
  \n 
  "}" '(c-indent-command) \n
  >  "else" \n
  "{" '(c-indent-command) \n
  \n 
  "}" '(c-indent-command) \n
)

;----------------------------------------------------------------------
(define-skeleton dino-try-catch-skeleton
  "Inserts a try/catch statement"
  "Which exception? "
  > "try" \n
  "{" '(c-indent-command) \n
  > _ \n 
  "} " '(c-indent-command) \n
  > "catch (" str " &exc)" \n
  "{" '(c-indent-command) \n 
  @ \n
  "} " '(c-indent-command) \n
  )

(define-skeleton dino-auto_ptr_skeleton
  "Inserts a stl auto_ptr "
  nil
  "std::auto_ptr<" _ ">")


;;----------------------------------------------------------------------
;; C++ typcast-skeletons 
;;----------------------------------------------------------------------

(define-skeleton dino-const_cast-skeleton
  "Inserts a c++ const_cast"
  "Type: "
  "dynamic_cast<" str ">(" _ ")"
  )

(define-skeleton dino-dynamic_cast-skeleton
  "Inserts a c++ dynamic_cast"
  "Type: "
  "dynamic_cast<" str ">(" _ ")"
  )

(define-skeleton dino-reinterpret_cast-skeleton
  "Inserts a c++ reinterpret_cast"
  "Type: "
  "reinterpret_cast<" str ">(" _ ")"
  )

(define-skeleton dino-static_cast-skeleton
  "Inserts a c++ static_cast"
  "Type: "
  "static_cast<" str ">(" _ ")"
  )

;----------------------------------------------------------------------
(define-skeleton dino-stl-container-skeleton
  "Inserts a stl container"
  nil
  > "std::" 
	(skeleton-read '(completing-read
                   "Container Type: " dino-stl-container-alist
                   )) 
  "<" 
  (skeleton-read "Element Type: ")
  ">")

;----------------------------------------------------------------------
(define-skeleton dino-iterate-stl-container-skeleton
  "Inserts a statement which iterates over a stl collection"
  nil
  (dino-stl-container-skeleton)
  "::const_iterator pos;" \n \n
  '(setq v1 (skeleton-read "Container Name: "))
  > "for (pos = " v1 ".begin(); pos!= " v1 ".end(); ++pos)" \n
  "{" '(c-indent-command) \n
  > _ \n 
  "}" '(c-indent-command) \n
  )

;;----------------------------------------------------------------------
;; Skeltons for Log4cpp Makros
;;----------------------------------------------------------------------

(define-skeleton dino-out-info-skeleton
"Inserts a Info Makro"
nil  
> "DOUT_INFO(\""
(skeleton-read "Category: ")
"\", \""
(skeleton-read "Message: ")
"\"" _");" \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-out-error-skeleton
"Inserts a Error Makro"
nil  
> "DOUT_ERROR(\""
(skeleton-read "Category: ")
"\", \""
(skeleton-read "Message: ")
"\""_");" \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-out-warn-skeleton
"Inserts a Warn Makro"
nil  
> "DOUT_WARN(\""
(skeleton-read "Category: ")
"\", \""
(skeleton-read "Message: ")
"\""_");" \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-out-debug-skeleton
"Inserts a Debug Makro"
nil  
> "DOUT_DEBUG(\""
(skeleton-read "Category: ")
"\", \""
(skeleton-read "Message: ")
"\""_");" \n
)

;;----------------------------------------------------------------------
;; Skeltons for C++ Class definitions
;;----------------------------------------------------------------------

(define-skeleton dino-class-skeleton
  "Inserts a simple c++ class definition "
  nil
  '(setq v1 (skeleton-read "Class: "))
  (dino-asterisk-skeleton)
  "/// " v1 ?\n
  "///"  ?\n
  "/// @author " (user-full-name) " <" dino-skeletons-user-mail-address ">" ?\n
  "/// @version 1.0.0" \n ?\n
  > "class " v1 \n
  "{\npublic:" '(c-indent-command) \n ?\n
  ;;  Constructor:
  > v1 "();" \n
  ;;  Destructor:
  > "virtual ~" v1 "();" \n
  >  _ \n
  "private:" '(c-indent-command) \n
  ;;  Copy Constructor:
  > v1 "(" v1 " const & src);" \n 
  ;;  Assignment Operator:
  > v1 "& operator = (" v1 " const & src);" \n \n
  "}; " comment-start "class " v1 " ends here" '(c-indent-command) \n \n
  )

;;----------------------------------------------------------------------
(define-skeleton dino-c++-header-class-skeleton
  "Inserts c++ class definition"
    nil
		?\n
		(dino-asterisk-skeleton)
    '(setq v1 (skeleton-read "Class name: "  
           (capitalize (substring (file-name-nondirectory buffer-file-name)
                                  0 (string-match "\\.[^.]*$"
                                                  (file-name-nondirectory buffer-file-name))))
           ))
    "///"  ?\n 
    "/// " v1 ?\n
    "///" ?\n
		"/// "
		'(setq v2 (skeleton-read "Class description: "))
		'(if (string= v2 "")
				 (insert "<insert Class description here>") 
			 (insert v2)
		 )  ?\n "///" ?\n
		"/// @author " (user-full-name) " <" dino-skeletons-user-mail-address ">" ?\n
		"/// @version "
		(skeleton-read "Version: " "1.0.0") ?\n
		"/// @see " ?\n
		"/// @invariant " ?\n
		"/// @ingroup "
		(skeleton-read "Module: " dino-skeletons-doxygen-group ) ?\n ?\n
    > "class "v1
   '(setq v2 (skeleton-read "Derived from: "))
    (if (null (string= v2 ""))
        (concat " : public " v2))
    \n
    "{\nprivate:" '(c-indent-command) \n
    > v1 "(" v1 " const & src);" \n 
    > v1 "& operator = (" v1 " const & src);" \n \n
    "public:" '(c-indent-command) \n ?\n
    (dino-minus-skeleton)
    "///"  ?\n
		"/// Default Constructor" ?\n
    "///"  ?\n ?\n
    > v1 "();" \n \n
    > "virtual ~" v1 "();" 
    \n \n
     "}; " comment-start "class " v1 " ends here" '(c-indent-command) \n
    )


;;----------------------------------------------------------------------
(define-skeleton dino-doxymacs-C++-function-comment-skeleton
	"Default C++-style template for function documentation."
	"Description: "
	?\n
	(dino-minus-skeleton)
	"/// " ?\n
	"/// " str ?\n
	'(if (fboundp 'doxymacs-find-next-func)
			 (setq v1 (doxymacs-find-next-func))
		 (setq v1 nil)
		 (message "Warning function doxymacs-find-next-func is not available, please install doxymacs!"))
	'(setq v2	(cdr (assoc 'args v1)))
	"///" ?\n
	"/// @return " (cdr (assoc 'return v1)) ?\n
	'(while v2
		 (insert (concat "/// @param " (car v2) " \n"))
		 (setq v2	(cdr v2)))
	"/// @pre " ?\n
	"/// @post " ?\n
  "/// @exception " ?\n \n
	)

;;----------------------------------------------------------------------
(define-skeleton dino-function-declaration-skeleton
	"Inserts a function declaration."
	nil
 	(if (y-or-n-p "virtual? ")
			(identity "virtual "))
	'(setq v2 (skeleton-read "Return Type: "))
	'(if (string= v2 "")
			(insert "void") 
		 (insert v2)
		 )
	" "
	'(setq v2 (skeleton-read "Function Name: "))
	> v2 "("
	'(setq v1 t)
	( "Parameter Type: " str " "
		'(setq v1 nil) ;; set if function has params
		'(setq v2  (skeleton-read "Parameter Name: "))
		> v2 ", ")	
	(if (not v1) -2) ;; if function has no params delete trailing ","
	> ")"
 	(if (y-or-n-p "is function const? ")
			(identity " const"))
	> ";" '(c-indent-command) \n
)

;;----------------------------------------------------------------------
(define-skeleton dino-main-function-skeleton
  "Inserts a c/c++ main function"
	nil
	>"int main(int argc,char *argv[])"\n
  "{" '(c-indent-command) \n
  > _ \n 
 "  return(0);" '(c-indent-command) \n
  "}" '(c-indent-command) \n
)

;######################################################################
;; pure c skeletons
;######################################################################

(defvar dino-include-c-alist
  '(("assert.h" . nil)	
    ("ctype.h"  . nil)	
    ("errno.h"  . nil)	
    ("float.h"  . nil)	
    ("iso646.h" . nil)	
    ("limits.h" . nil)	
    ("locale.h" . nil)	
    ("math.h"   . nil)	
    ("setjmp.h" . nil)	
    ("signal.h" . nil)	
    ("stdarg.h" . nil)	
    ("stddef.h" . nil)	
    ("stdio.h"  . nil)	
    ("stdlib.h" . nil)	
    ("string.h" . nil)	
    ("time.h"   . nil)	
    ("wchar.h"  . nil)	
    ("wtype.h"  . nil))
    "Alist for dino-include-c-skeleton")

;;---------------------------------------------------------------------
(define-skeleton dino-include-c-skeleton
	"Inserts a C include macro"
	nil
	"#include <"
	(skeleton-read '(completing-read
                   "Standard C Header: " dino-include-c-alist
											 ))
	">" ?\n
  )

;;----------------------------------------------------------------------
(define-skeleton dino-define-skeleton
	"Inserts a c #define macro" 
	nil 
	"#define "
	(skeleton-read "identifier: " ) " "
	(skeleton-read "replacement: " ) ?\n
  )

(provide 'dino-cc-mode-skeletons)
;;; dino-cc-mode-skeletons.el ends here
