;;; dino-latex-init.el --- Configuration of the dino latex environment
;	$Id: dino-latex-init.el,v 1.15 2004/10/28 08:16:19 sthalau Exp $	

;; Copyright (C) 2003  IICM, Graz University of Technology
;; Inffeldgasse 16c, A-8010 Graz, Austria.

;; Author: stefan thalauer <sthalau@iicm.edu>
;; Keywords: wp, tex

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

;; (setq debug-on-error t)

(require 'tex-site)
(require 'dino-skeletons)

;;----------------------------------------------------------------------
;; AUCTeX Settings

(setq TeX-auto-save nil
      TeX-parse-self t
			TeX-insert-braces nil    ; append no empty pair of braces after inserting a macro
			TeX-electric-escape nil  ; bound \ to TeX-electric-macro
      LaTeX-default-environment "itemize"
      LaTeX-section-list 
      '(
        ("part" 0)
        ("chapter" 1)
        ("section" 2)
        ("subsection" 3)
        ("subsubsection" 4)
        ("subsubsubsection" 5)
        ("paragraph" 6)
        ("subparagraph" 7)
        ("subsubparagraph" 8)
        ("usecase" 9)
        ("userrequirement" 10)
        ("subuserrequirement" 11)
        ("subsubuserrequirement" 12)
        ("softwarerequirement" 13)
        ("subsoftwarerequirement" 14)
        ("subsubsoftwarerequirement" 15)
        ("softwaredesign" 16)
        ("subsoftwaredesign" 17)
        ("subsubsoftwaredesign" 18))
      )

; FIXXME (Stefan Thalauer, 2004-08-16 13:35:30) -> Maybe obsolete in AUCTeX 11.50?

;; (setq LaTeX-left-comment-regexp "%[^%%]"
;;       LaTeX-right-comment-regexp "%%%"

; END FIXXME (Stefan Thalauer, 2004-08-16 13:35:30)

(setq-default TeX-master nil)

;;------------------------------ Users private libaries 
                                        ;TeX-macro-private '("~/lib/tex-lib/")
(let ((dino-localroot (getenv dino-environment-localroot)))
	(if dino-localroot
			(setq TeX-style-private                                  ; AUC-TeX-Macros
						(list "~/lib/site-lisp/auctex/style/" 
									(concat dino-localroot 
                          "/dinopolis/dev_environment/emacs/lib/site-lisp/auctex/style/")))
		(setq TeX-style-private "~/lib/site-lisp/auctex/style/")))				
(setq TeX-auto-private "~/lib/site-lisp/auctex/auto/") ; Autom. Auc-TeX-Macros

;; ;;------------------------------ LateX-mode-hook
(add-hook 'LaTeX-mode-hook 
          (lambda () (abbrev-mode) t))
(add-hook 'LaTeX-mode-hook 
          (lambda () (setq local-abbrev-table latex-mode-abbrev-table)))
(add-hook 'LaTeX-mode-hook 'flyspell-mode)
(add-hook 'LaTeX-mode-hook 
          (lambda () (eval (ispell-change-dictionary "english"))))

(add-hook 'LaTeX-mode-hook 
          (lambda () 
						(TeX-load-style "dino_sty")
						))


(add-hook 'LaTeX-mode-hook 
          (lambda () 
						(TeX-load-style "iicm_sty")
						))

(add-hook 'LaTeX-mode-hook 
          (lambda () 
            (TeX-run-style-hooks "dino_sty")
						))


(add-hook 'LaTeX-mode-hook 
          (lambda () 
            (TeX-run-style-hooks "iicm_sty")
						))

; FIXXME (Stefan Thalauer, 2004-08-16 13:46:56) -> Obsolete in AUCTeX 11.50

;; ;;------------------------------ Additional TeX-command-master commands:
;; (defvar dino-pdf-viewer "xpdf" 
;; 	"Default view-command for PDF files.") 
;; (defvar dino-pdf-expand-string "%x") 

;; ;; Recognize pdf files: 
;; (add-hook 'LaTeX-mode-hook
;; 					(lambda ()
;; 						(add-to-list 'TeX-expand-list (list dino-pdf-expand-string 'file "pdf" t) t)))

;; ;; "View Pdf" : View the pdf file with the command defined in `dino-pdf-viewer'
;; (add-hook 'LaTeX-mode-hook
;; 					(lambda ()
;; 						(add-to-list 'TeX-command-list 
;; 												 (if (or window-system (getenv "DISPLAY"))
;; 														 (list "View PDF" 
;; 																	 (concat dino-pdf-viewer " " dino-pdf-expand-string)
;; 																	 'TeX-run-silent t nil)
;; 													 (list "View PDF" (concat "pdftotext " dino-pdf-expand-string)
;; 																 'TeX-run-command t nil)) t)))

;; (add-hook 'LaTeX-mode-hook
;; 					(lambda ()
;; 						(add-to-list 'TeX-command-list 
;; 												 (list "2pages" (concat "doublepdfpages " dino-pdf-expand-string) 
;; 															 'TeX-run-command t nil)) t))

;; ;; run make
;; (add-hook 'LaTeX-mode-hook
;; 					(lambda ()
;; 						(add-to-list 'TeX-command-list 
;; 												 (list "Make" "make " 
;; 															 'TeX-run-compile t nil)) t))
;; ;; run make dvi
;; (add-hook 'LaTeX-mode-hook
;; 					(lambda ()
;; 						(add-to-list 'TeX-command-list 
;; 												 (list "Make dvi" "make %d" 
;; 															 'TeX-run-compile t nil)) t))


;; ;; `Clean': remove all temporary  latex files 
;; (add-hook 'LaTeX-mode-hook
;; 					(lambda ()
;; 						(add-to-list 'TeX-command-list (list "Clean" "rm *.aux *.bbl *.blg *.idx *.ilg *.ind *.toc *.log *.log *.brf *.out *.lof *.lot *.gxg *.glx *.gxs *.glo *.gls *.dinogenslinfo *.dinoslides *.dinoslidestmp" 'TeX-run-command t nil) t)))


; END FIXXME (Stefan Thalauer, 2004-08-16 13:46:56)

;; ;**********************************************************************
(defun dino-LaTeX-item ()
  "Inserts item with leading line of dots"
  (dino-dot-skeleton)
  (TeX-insert-macro "item"))

(defun dino-LaTeX-item-argument ()
  "Insert a new item with an optional argument and a leading line of dots"
  (let ((TeX-arg-item-label-p t))
		(dino-dot-skeleton)
    (TeX-insert-macro "item")))

;; ;**********************************************************************
(defvar dino-Latex-chapter-seperator 'dino-sharp-skeleton "Seperator skeleton for chapters")
(defvar dino-Latex-section-seperator 'dino-sharp-skeleton "Seperator skeleton for sections")
(defvar dino-Latex-subsection-seperator 'dino-sharp-skeleton "Seperator skeleton for subsections")
(defvar dino-Latex-subsubsection-seperator 'dino-minus-skeleton "Sepertor skeleton for subsubsections")
(defvar dino-Latex-default-seperator 'dino-minus-skeleton "Default Seperator skeleton")

;; ;;seperator hook
(defun dino-Latex-section-seperator-function ()  
  "Hook which inserts the seperator for a LaTex section.
Insert this hook into `LaTeX-section-hook' to insert the seperator of
the section inserted with `\\[LaTeX-section]."
  (cond 
   ((= level 1) ; chapter
    (let ((count 0)) 
      (while (< count 4)
				(funcall dino-Latex-chapter-seperator)
        (setq count (1+ count)))
      ))
   ((= level 2) ; section
    (let ((count 0)) 
      (while (< count 2)
				(funcall dino-Latex-section-seperator)
        (setq count (1+ count)))
    ))
   ((= level 3) ; subsection
		(funcall dino-Latex-subsection-seperator)
    )
   ((= level 4) ; subsubsection
    (let ((count 0)) 
      (while (< count 2)
				(funcall dino-Latex-subsubsection-seperator)
        (setq count (1+ count)))
    ))
   ((> level 4)  ; default
		(funcall dino-Latex-default-seperator)
    )
   ))

(setq LaTeX-section-hook
      '(
        LaTeX-section-heading
        dino-Latex-section-seperator-function
        LaTeX-section-toc
        LaTeX-section-title
        LaTeX-section-section
        LaTeX-section-label
        ))

;;----------------------------------------------------------------------
;; Reftex settings
;;----------------------------------------------------------------------
(autoload 'reftex-mode    "reftex" "RefTeX Minor Mode" t)                    
(autoload 'turn-on-reftex "reftex" "RefTeX Minor Mode" nil)                  

(add-hook 'LaTeX-mode-hook 'turn-on-reftex)

(setq reftex-plug-into-AUCTeX t
      reftex-max-section-depth 20
      reftex-insert-label-flags '("sf" "sft")
			reftex-comment-citations nil
			reftex-view-crossref-extra '(("stdref" %s 0))
      ;; for RefTeX version > 4.16 
      reftex-include-file-commands '("include" "input" "includedoc")
      reftex-section-levels 
      '(
        ("part" . 0)
        ("chapter" . 1)
        ("section" . 2)
        ("subsection" . 3)
        ("subsubsection" . 4)
        ("subsubsubsection" . 5)
        ("paragraph" . 6)
        ("subparagraph" . 7)
        ("subsubparagraph" . 8)
        ("usecase" . 9)
        ("userrequirement" . 10)
        ("subuserrequirement" . 11)
        ("subsubuserrequirement" . 12)
        ("softwarerequirement" . 13)
        ("subsoftwarerequirement" . 14)
        ("subsubsoftwarerequirement" . 15)
        ("softwaredesign" . 16)
        ("subsoftwaredesign" . 17)
        ("subsubsoftwaredesign" . 18)
;;         ("addchap" . -1)
;;         ("addsec" . -2)
        )
      reftex-section-prefixes 
      '(( 0 . "part:")
        ( 1 . "chap:")
        ( 2 . "sect:")
        ( 3 . "ssect:")
        ( 4 . "sssect:")
        ( 5 . "ssssect:")
        ( 6 . "par:")
        ( 7 . "spar:")
        ( 8 . "sspar:")
        ( 9 . "uc:")
        (10 . "ur:")
        (11 . "sur:")
        (12 . "ssur:")
        (13 . "sr:")
        (14 . "ssr:")
        (15 . "sssr:")
        (16 . "sd:")
        (17 . "ssd:")
        (18 . "sssd:")
        (t  . "sec:")
        )
      ;; TOC: Table of Contents
      reftex-toc-include-file-boundaries t
      reftex-toc-include-labels t
      )

;; ;;------------------------------ INDEX
;; (setq reftex-index-macros
;; 			'(("\\ac{*}" "ac" ?a "" nil nil)
;; 				("\\gls{*}" "glo" ?G "" nil nil)
;; 				))


;; ;;-------------------------------------- autoinserting latex template
;; (mapcar
;;  '(lambda (auto)
;; 		(setq auto-insert-alist
;;           (cons auto
;;                 (delq (assoc (car auto) auto-insert-alist)
;;                       auto-insert-alist))))
;;  '(
;;    ((latex-mode . "AUCTeX skeleton") nil (dino-latex-template)
;;     )
;;    )
;;  )

;; (define-skeleton dino-latex-template
;;   "Inserts a header for a c++ class"
;;   "Document Title: "
;;   "%% " (file-name-nondirectory (buffer-file-name)) " --- " str \n \n
;;   "%% This article following the Dino Documentation Guidelines.

;; \\documentclass[a4paper]{article}
;; \\usepackage[dinodraft]{dino}

;; \\begin{document}
;; %% ======================================== start document header

;; \\title{" str  "}

;; \\author{\\iicm \\dinopolis Team\\\\
;;    (\\" dino-username ")\\\\         
;;    $Revision: 1.15 $}                 

;; \\maketitle

;; \\vspace{1cm}

;; \\begin{abstract}

;;   %% insert abstract here

;; \\end{abstract}

;; \\newpage

;; \\tableofcontents

;; \\newpage

;; %% list of acronyms
;; %\\addcontentsline{toc}{chapter}{\\listacronymname}
;; %\\printglosstex(acr)

;; %% ======================================== end document header

;; %% ======================================== begin document body
;; "
;; _

;; "
;; %% ======================================== end document body

;; %% glossary
;; %\\addcontentsline{toc}{chapter}{\\glossaryname}
;; %\\printglosstex(glo)

;; %% ======================================== begin references

;; %\\bibliographystyle{alpha}
;; %\\addcontentsline{toc}{chapter}{\\bibname}
;; %\\bibliography{../../bibliography_entries} 

;; %% ======================================== end references

;; %% end of document marker to be able to see if the document is
;; %% complete when printed

;; \\documentend

;; \\end{document}"
;;   )

;;; dino-latex-init.el ends here
