;;; dino-skeletons.el --- Skeletons of the Dinopolis project group
;	$Id: dino-skeletons.el,v 1.52 2004/09/08 14:19:59 sthalau Exp $	

;; Copyright (C) 2003  IICM, Graz University of Technology
;; Inffeldgasse 16c, A-8010 Graz, Austria.

;; Author: stefan thalauer  <sthalau@iicm.edu>
;; Keywords: wp ,local, c


;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Copy this file in your ~/lib/lisp directory

;;; Code:

(require 'skeleton)
(require 'copyright)

(defvar dino-skeletons-address "Inffeldgasse 16c, A-8010 Graz, Austria.")
(defvar dino-skeletons-user-mail-address user-mail-address "Dino Mailadress of user") 
(defvar dino-skeletons-fixme-string "FIXXME") 

;;----------------------------------------------------------------------
(define-skeleton dino-timestamp-skeleton
  "Inserts a timestamp"
  nil
  comment-start 
  "Time-stamp: <" 
  (format-time-string "%Y-%m-%d %T") " " 
  (user-login-name) "> " comment-end \n)

;;----------------------------------------------------------------------
(define-skeleton dino-date-skeleton
  "Inserts the current date "
  nil
  (format-time-string "%Y-%m-%d %T"))

;;----------------------------------------------------------------------
(define-skeleton dino-fixxme-skeleton
  "Inserts a fixxme comment"
  "Description: "
   '(setq v1 (concat " (" (user-full-name) ", " 
                     (format-time-string "%Y-%m-%d %T") ")"))
   comment-start 
   " " dino-skeletons-fixme-string
   v1 " -> " str 
   comment-end 
   \n ?\n
   > _ 
   \n ?\n
   comment-start 
   " END " dino-skeletons-fixme-string
   v1
   comment-end ?\n)

;;----------------------------------------------------------------------
(define-skeleton dino-minus-skeleton
  "Inserts a comment line :
 \"----------------------------------------------------------------------\""
  nil
  comment-start 
  "----------------------------------------------------------------------" 
  comment-end ?\n)

;;----------------------------------------------------------------------
(define-skeleton dino-asterisk-skeleton
  "Inserts a comment line :
 \"**********************************************************************\"" 
  nil
  comment-start 
  "**********************************************************************" 
  comment-end ?\n)


;;----------------------------------------------------------------------
(define-skeleton dino-sharp-skeleton
  "Inserts a comment line :
 \"######################################################################\"" 
  nil
  comment-start 
  "######################################################################" 
  comment-end ?\n)

;;----------------------------------------------------------------------
(define-skeleton dino-dot-skeleton
  "Inserts a comment line :
 \"......................................................................\"" 
  nil
  comment-start 
  "......................................................................" 
  comment-end ?\n)

;;----------------------------------------------------------------------
(define-skeleton dino-copyright-skeleton
  "Inserts the iicm copyright message"
  nil
  (dino-asterisk-skeleton)
  (copyright)   \n
  comment-start dino-skeletons-address comment-end \n
  comment-start " " comment-end \n
  comment-start "This program is free software; you can redistribute it and/or modify" comment-end \n
  comment-start "it under the terms of the GNU Lesser General Public License (LGPL)" comment-end \n
  comment-start "as published by the Free Software Foundation; either version 2.1 of" comment-end \n
  comment-start "the License, or (at your option) any later version." comment-end \n
  comment-start " " comment-end \n
  comment-start "This program is distributed in the hope that it will be useful," comment-end \n
  comment-start "but WITHOUT ANY WARRANTY; without even the implied warranty of" comment-end \n
  comment-start "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the" comment-end \n
  comment-start "GNU Lesser General Public License for more details." comment-end \n
  comment-start " " comment-end \n
  comment-start "You should have received a copy of the GNU Lesser General Public " comment-end \n
  comment-start "License along with this program; if not, write to the" comment-end \n
  comment-start "Free Software Foundation, Inc., " comment-end \n
  comment-start "59 Temple Place, Suite 330, Boston, MA  02111-1307 USA" comment-end \n
  (dino-asterisk-skeleton) ?\n
)

;;**********************************************************************
;; Autoinsert  
;;**********************************************************************

;;---------------------------------------------------------------------
(define-skeleton dino-file-shortdescription-skeleton 
	"Insert a short description about the file"
	"Short description: "
	comment-start " "
	(file-name-nondirectory buffer-file-name)
	" - " str  " " 
  comment-end \n 
	)

;;---------------------------------------------------------------------
(define-skeleton dino-simple-header-skeleton
  "Inserts a header for a file"
	nil
	(dino-file-shortdescription-skeleton) \n
	(dino-timestamp-skeleton) \n \n
	comment-start "Author: " (user-full-name) " <" dino-skeletons-user-mail-address ">" \n 
	\n _
	)

;;---------------------------------------------------------------------
(define-skeleton dino-url-skeleton
  "Inserts a url"
  "URL: "
  "<URL:" str ">")

;;---------------------------------------------------------------------
(define-skeleton dino-tag-skeleton
"Inserts a html or xml tag"
"Tag: "
"<" str _ "></" str ">" )


(provide 'dino-skeletons)
;;; dino_skeletons.el ends here


